/*
 * Copyright (C) 2011 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "smmorphoutputview.hh"
#include "smmorphplan.hh"
#include <birnet/birnet.hh>

using namespace SpectMorph;

using std::string;
using std::vector;

namespace {

struct MyOperatorFilter : public OperatorFilter
{
  bool filter (MorphOperator *op)
  {
    return (op->output_type() == MorphOperator::OUTPUT_AUDIO);
  }
} op_filter_instance;

}

MorphOutputView::MorphOutputView (MorphOutput *morph_output, MorphPlanWindow *morph_plan_window) :
  MorphOperatorView (morph_output, morph_plan_window),
  morph_output (morph_output)
{
  for (int ch = 0; ch < 4; ch++)
    {
      ChannelView *chv = new ChannelView (morph_output->morph_plan(), &op_filter_instance);
      chv->label.set_text (Birnet::string_printf ("Channel #%d", ch + 1));
      channel_table.attach (chv->label, 0, 1, ch, ch + 1, Gtk::SHRINK);
      channel_table.attach (chv->combobox, 1, 2, ch, ch + 1);
      channels.push_back (chv);

      chv->combobox.set_active (morph_output->channel_op (ch));
    }

  for (int ch = 0; ch < 4; ch++)
    channels[ch]->combobox.signal_active_changed.connect (sigc::mem_fun (*this, &MorphOutputView::on_operator_changed));

  channel_table.set_spacings (10);
  channel_table.set_border_width (5);
  frame.add (channel_table);

  show_all_children();
}

MorphOutputView::~MorphOutputView()
{
  for (vector<ChannelView *>::iterator i = channels.begin(); i != channels.end(); i++)
    {
      delete *i;
    }
}

void
MorphOutputView::on_operator_changed()
{
  for (size_t i = 0; i < channels.size(); i++)
    morph_output->set_channel_op (i, channels[i]->combobox.active());
}
